@@ -7,7 +7,7 @@ function prepare_pxe_pxelinux {
         cp /usr/lib/PXELINUX/pxelinux.0 \
            /usr/lib/syslinux/modules/bios/{chain.c32,ldlinux.c32} \
            ${TFTPBOOT_PATH}/
-    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos ]]; then
+    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos|rocky ]]; then
         if [[ "${TFTPBOOT_PATH}" != /tftpboot ]]; then
             cp /tftpboot/{pxelinux.0,chain.c32,ldlinux.c32} \
                ${TFTPBOOT_PATH}/
@@ -20,9 +20,9 @@ function prepare_pxe_grub {
     if [[ "${KOLLA_BASE_DISTRO}" =~ debian|ubuntu  ]]; then
         shim_src_file="/usr/lib/shim/shim*64.efi.signed"
         grub_src_file="/usr/lib/grub/*-efi-signed/grubnet*64.efi.signed"
-    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos ]]; then
-        shim_src_file="/boot/efi/EFI/centos/shim*64.efi"
-        grub_src_file="/boot/efi/EFI/centos/grub*64.efi"
+    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos|rocky ]]; then
+        shim_src_file="/boot/efi/EFI/${KOLLA_BASE_DISTRO}/shim*64.efi"
+        grub_src_file="/boot/efi/EFI/${KOLLA_BASE_DISTRO}/grub*64.efi"
     fi
 
     if [[ "${KOLLA_BASE_ARCH}" == "x86_64" ]]; then
@@ -43,18 +43,19 @@ function prepare_ipxe {
     # was ipxe.efi. Ensure that both exist, using symlinks where the files are
     # named differently to allow the original names to be used in ironic.conf.
     if [[ "${KOLLA_BASE_DISTRO}" =~ debian|ubuntu ]]; then
-        cp /usr/lib/ipxe/{undionly.kpxe,ipxe.efi} ${TFTPBOOT_PATH}/
-        # NOTE(mgoddard): The 'else' can be removed  when snponly.efi is
-        # available in Jammy 22.04.
-        if [[ -f /usr/lib/ipxe/snponly.efi ]]; then
-            cp /usr/lib/ipxe/snponly.efi ${TFTPBOOT_PATH}/snponly.efi
-        elif [[ ! -e ${TFTPBOOT_PATH}/snponly.efi ]]; then
-            ln -s ${TFTPBOOT_PATH}/ipxe.efi ${TFTPBOOT_PATH}/snponly.efi
+        # NOTE(m-anson): ipxe-arm64.efi is not symlinked from /boot to
+        # /usr/lib/ipxe by the Ubuntu ipxe package, so fix that here.
+        if [[ -e /boot/ipxe-arm64.efi ]]; then
+            ln -s /boot/ipxe-arm64.efi /usr/lib/ipxe/
         fi
-    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos ]]; then
+        cp /usr/lib/ipxe/{undionly.kpxe,ipxe*.efi,snponly.efi} ${TFTPBOOT_PATH}/
+    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos|rocky ]]; then
         cp /usr/share/ipxe/{undionly.kpxe,ipxe*.efi} ${TFTPBOOT_PATH}/
+        cp /usr/share/ipxe/arm64-efi/snponly.efi ${TFTPBOOT_PATH}/ipxe-snponly-aarch64.efi
         if [[ ! -e ${TFTPBOOT_PATH}/ipxe.efi ]]; then
-            ln -s ${TFTPBOOT_PATH}/ipxe-${KOLLA_BASE_ARCH}.efi ${TFTPBOOT_PATH}/ipxe.efi
+            # NOTE(m-anson): No ipxe-aarch64.efi exists so no need to use
+            # $KOLLA_BASE_ARCH in the symlink target
+            ln -s ${TFTPBOOT_PATH}/ipxe-x86_64.efi ${TFTPBOOT_PATH}/ipxe.efi
         fi
         if [[ ! -e ${TFTPBOOT_PATH}/snponly.efi ]]; then
             ln -s ${TFTPBOOT_PATH}/ipxe-snponly-${KOLLA_BASE_ARCH}.efi ${TFTPBOOT_PATH}/snponly.efi
@@ -62,6 +63,35 @@ function prepare_ipxe {
     fi
 }
 
+function prepare_esp_image {
+    # NOTE(bbezak): based on https://docs.openstack.org/ironic/2024.2/install/configure-esp.html
+    # ESP image needs to be provided for UEFI boot with virtual media:
+    # https://docs.openstack.org/ironic/2024.2/admin/drivers/redfish.html#virtual-media-boot
+    if [[ "${KOLLA_BASE_DISTRO}" =~ debian|ubuntu ]]; then
+        shim_src_file="/usr/lib/shim/shim*64.efi.signed"
+        grub_src_file="/usr/lib/grub/*-efi-signed/grubnet*64.efi.signed"
+    elif [[ "${KOLLA_BASE_DISTRO}" =~ centos|rocky ]]; then
+        shim_src_file="/boot/efi/EFI/${KOLLA_BASE_DISTRO}/shim*64.efi"
+        grub_src_file="/boot/efi/EFI/${KOLLA_BASE_DISTRO}/grub*64.efi"
+    fi
+
+    if [[ "${KOLLA_BASE_ARCH}" == "x86_64" ]]; then
+        shim_dst_file="bootx64.efi"
+        grub_dst_file="grubx64.efi"
+    elif [[ "${KOLLA_BASE_ARCH}" == "aarch64" ]]; then
+        shim_dst_file="bootaa64.efi"
+        grub_dst_file="grubaa64.efi"
+    fi
+
+    DEST=${HTTPBOOT_PATH}/esp.img
+    dd if=/dev/zero of=$DEST bs=4096 count=2048
+    mkfs.msdos -F 12 -n ESP_IMAGE $DEST
+    mmd -i $DEST EFI EFI/BOOT
+    mcopy -i $DEST -v $shim_src_file ::EFI/BOOT/$shim_dst_file
+    mcopy -i $DEST -v $grub_src_file ::EFI/BOOT/$grub_dst_file
+    mdir -i $DEST ::EFI/BOOT
+}
+
 # Bootstrap and exit if KOLLA_BOOTSTRAP variable is set. This catches all cases
 # of the KOLLA_BOOTSTRAP variable being set, including empty.
 if [[ "${!KOLLA_BOOTSTRAP[@]}" ]]; then
@@ -70,6 +100,7 @@ if [[ "${!KOLLA_BOOTSTRAP[@]}" ]]; then
     prepare_pxe_pxelinux
     prepare_pxe_grub
     prepare_ipxe
+    prepare_esp_image
     exit 0
 fi
 
