@@ -141,6 +141,7 @@ class KollaWorker(object):
         self.image_statuses_unbuildable = dict()
         self.image_statuses_allowed_to_fail = dict()
         self.maintainer = conf.maintainer
+        self.patches_path = conf.patches_path
 
         try:
             self.engine_client = engine.getEngineClient(self.conf)
@@ -270,6 +271,7 @@ class KollaWorker(object):
         return {
             'debian_package_install': jinja_methods.debian_package_install,
             'handle_repos': jinja_methods.handle_repos,
+            'raise_error': jinja_methods.raise_error,
         }
 
     def get_users(self):
@@ -279,7 +281,17 @@ class KollaWorker(object):
         for section in all_sections:
             match = re.search('^.*-user$', section)
             if match:
-                user = self.conf[match.group(0)]
+                cfg_group_name = match.group(0)
+
+                if cfg_group_name not in self.conf._groups:
+                    self.conf.register_opts(
+                        common_config.get_user_opts(
+                            None, None,
+                            # cut `-user` suffix
+                            group=cfg_group_name[:-5]),
+                        group=cfg_group_name
+                    )
+                user = self.conf[cfg_group_name]
                 ret[match.group(0)[:-5]] = {
                     'uid': user.uid,
                     'gid': user.gid,
@@ -322,7 +334,8 @@ class KollaWorker(object):
                       'distro_package_manager': self.distro_package_manager,
                       'rpm_setup': self.rpm_setup,
                       'build_date': build_date,
-                      'clean_package_cache': self.clean_package_cache}
+                      'clean_package_cache': self.clean_package_cache,
+                      'patches_path': self.patches_path}
             env = jinja2.Environment(  # nosec: not used to render HTML
                 loader=jinja2.FileSystemLoader(self.working_dir))
             env.filters.update(self._get_filters())
@@ -349,6 +362,35 @@ class KollaWorker(object):
                 f.write(content)
                 LOG.debug("Wrote it to %s", content_path)
 
+    def create_patch_files(self):
+        for path in self.docker_build_paths:
+            image_name = os.path.basename(path)
+            src_patch_folder = os.path.join(self.patches_path, image_name) if \
+                self.patches_path and os.path.isdir(self.patches_path) \
+                else None
+            dest_patch_folder = os.path.join(path, "patches")
+            os.makedirs(dest_patch_folder, exist_ok=True)
+
+            if not src_patch_folder or not os.path.isdir(src_patch_folder):
+                continue
+
+            src_series_path = os.path.join(src_patch_folder, "series")
+            dest_series_path = os.path.join(dest_patch_folder, "series")
+
+            if os.path.isfile(src_series_path):
+                shutil.copyfile(src_series_path, dest_series_path)
+                with open(src_series_path) as f:
+                    patch_files = [line.strip() for line in f
+                                   if not line.startswith("#")]
+            else:
+                patch_files = []
+
+            for patch_file in patch_files:
+                src_patch_file = os.path.join(src_patch_folder, patch_file)
+                dest_patch_file = os.path.join(dest_patch_folder, patch_file)
+                if os.path.isfile(src_patch_file):
+                    shutil.copyfile(src_patch_file, dest_patch_file)
+
     def _merge_overrides(self, overrides):
         tpl_name = os.path.basename(overrides[0])
         with open(overrides[0], 'r') as f:
